package com.hubspot.sprinter;

import java.util.Collections;
import java.util.List;

import com.hubspot.utils.HubSpotIdentifiableObjectManager;
import com.hubspot.utils.db.DbUtils;
import com.hubspot.utils.db.HqlHelper;

/**
 * Manager for the Sprint object.
 * 
 * @author dabdinoor
 */
public class SprintManager extends HubSpotIdentifiableObjectManager {
    /** Get Sprint. */
    public Sprint get(final Long id) {
        return (Sprint) super.get(Sprint.class, id);
    }

    /** Update Sprint. */
    public boolean update(final Sprint sprint) {
        return super.update(sprint);
    }

    /** Delete Sprint. */
    public boolean delete(final Sprint sprint) {
        return super.delete(sprint);
    }

    /** Save Sprint. */
    public Sprint save(final Sprint sprint) {
        Long savedId = super.save(sprint);
        return get(savedId);
    }

    /** A more complicated type of save that sets up several objects within a sprint like teams and developers. */
    public boolean create(final Sprint sprint) {
        Long savedId = super.save(sprint);
        Sprint saved = get(savedId);
        if (saved == null) {
            return false;
        }

        // for each team, create a sprint team object
        List<Team> teams = ManagerUtils.getTeamManager().list(true);
        for (Team team : teams) {
            SprintTeam sprintTeam = new SprintTeam();
            sprintTeam.setSprint(saved);
            sprintTeam.setTeam(team);
            sprintTeam.setSprintDays(sprint.getDefaultDays());
            sprintTeam.setVelocity(team.getAverageVelocity());

            SprintTeam savedSprintTeam = ManagerUtils.getSprintTeamManager().save(sprintTeam);
            if (savedSprintTeam == null) {
                return false;
            }

            // create sprint team stats
            SprintTeamStats sprintTeamStats = new SprintTeamStats();
            sprintTeamStats.setSprint(saved);
            sprintTeamStats.setTeam(team);
            sprintTeamStats.setSprintTeam(savedSprintTeam);
            sprintTeamStats.setPlannedPoints(0);
            sprintTeamStats.setCompletedPoints(0);
            sprintTeamStats.setPlannedStories(0);
            sprintTeamStats.setCompletedStories(0);
            sprintTeamStats.setDeveloperDays(0.0);
            SprintTeamStats savedSprintTeamStats = ManagerUtils.getSprintTeamStatsManager().save(sprintTeamStats);
            if (savedSprintTeamStats == null) {
                return false;
            }

            // get developers for this team
            List<Developer> developers = ManagerUtils.getDeveloperManager().listByDefaultTeamId(team.getId());
            for (Developer developer : developers) {
                // put developer on team for this sprint
                SprintTeamDeveloper sprintTeamDeveloper = new SprintTeamDeveloper();
                sprintTeamDeveloper.setSprint(saved);
                sprintTeamDeveloper.setDeveloper(developer);
                sprintTeamDeveloper.setSprintTeam(sprintTeam);
                sprintTeamDeveloper.setPercentage(developer.getDefaultPercentage());
                sprintTeamDeveloper.setVacation(0.0);

                ManagerUtils.getSprintTeamDeveloperManager().save(sprintTeamDeveloper);
            }

        }

        return true;
    }

    /** A complicated method to destroy a sprint. Wipes out all of the dependent objects. */
    public boolean destroy(final Sprint sprint) {
        // get sprint teams
        List<SprintTeam> sprintTeams = ManagerUtils.getSprintTeamManager().listBySprintId(sprint.getId());
        for (SprintTeam sprintTeam : sprintTeams) {
            // delete sprint team developers
            List<SprintTeamDeveloper> sprintTeamDevelopers = ManagerUtils.getSprintTeamDeveloperManager().list(
                    sprintTeam.getId());
            for (SprintTeamDeveloper sprintTeamDeveloper : sprintTeamDevelopers) {
                ManagerUtils.getSprintTeamDeveloperManager().delete(sprintTeamDeveloper);
            }

            // delete stats
            List<SprintTeamStats> sprintTeamStats = ManagerUtils.getSprintTeamStatsManager().listBySprintTeamId(
                    sprintTeam.getId());
            for (SprintTeamStats stats : sprintTeamStats) {
                ManagerUtils.getSprintTeamStatsManager().delete(stats);
            }

            // delete sprint team
            ManagerUtils.getSprintTeamManager().delete(sprintTeam);
        }

        // delete the sprint
        return super.delete(sprint);
    }

    /** List Sprints. */
    public List<Sprint> list() {
        String hql = "FROM Sprint ORDER BY sequence DESC";

        HqlHelper helper = DbUtils.createHqlHelper(getSessionFactory());
        List<Sprint> result = helper.runHql(hql);
        if (result == null) {
            return Collections.EMPTY_LIST;
        }

        return result;
    }
}
