package com.hubspot.sprinter.web;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;

import com.hubspot.sprinter.ManagerUtils;
import com.hubspot.sprinter.SprintTeam;
import com.hubspot.sprinter.SprintTeamStats;

/**
 * Backs the sprint team plan screen.
 * 
 * @author dabdinoor
 */
public class TeamVelocityAction extends BaseAction {
    /** The ID of the sprint team being viewed. */
    private Long id;

    /** The sprint team. */
    private SprintTeam sprintTeam;

    /** The list of all sprint stats. */
    private List<SprintTeamStats> sprintTeamStats;
    
    /** velocity data. Converted to javascript inserts to 
     *  a Google Charts datatable by accessor
     */
    private List<Double> velocityValues;
    private List<Double> avgVelocityValues;
    private ArrayList<String> sprintNameValues;


    @Override
    public String execute() throws Exception {
        return SUCCESS;
    }

    /**
     * @return the id
     */
    public Long getId() {
        return id;
    }

    /**
     * @param id
     *            the id to set
     */
    public void setId(Long id) {
        this.id = id;
    }

    /**
     * @return the URL to google chart for team velocity.
     * @throws UnsupportedEncodingException
     */
    public String getVelocityChartURL() throws UnsupportedEncodingException {
        StringBuilder url = new StringBuilder(
                "http://chart.apis.google.com/chart?chs=450x200&cht=lc&chco=1E90FF,666666&chxt=x,y&chls=1|1,2,3");

        StringBuilder values = new StringBuilder();
        velocityValues = new ArrayList<Double>();
        avgVelocityValues = new ArrayList<Double>();
        sprintNameValues = new ArrayList<String>();
        
        StringBuilder average = new StringBuilder();

        // set up x-axis
        int xMax = 0;
        url.append("&chxl=0:|");

        // formatter for velocity values
        NumberFormat formatter = new DecimalFormat("#0.00");

        for (SprintTeamStats teamStats : getSprintTeamStats()) {
            String sprintName = teamStats.getSprint().getName();
            url.append(URLEncoder.encode(sprintName, "UTF-8"));
            url.append("|");
            
            // We pass a negative one to the front end to indicate that velocity isn't available
            // Sprints which actually have zero velocity will be interpolated over.
            // Boils down to completedPoints/developerDays, with truncation at 2 sig. digits .
            final Double velocity = 
                   Double.valueOf( 
                     formatter.format(
                      (teamStats.getDeveloperDays() > 0 && teamStats.getCompletedPoints() > 0) ?
                      teamStats.getCompletedPoints()
                      / teamStats.getDeveloperDays() : -1.0 
                       )
                      );

            final Double teamAverageVelocity = getSprintTeam().getTeam().getAverageVelocity();
            values.append(velocity).append(",");
            average.append(formatter.format(teamAverageVelocity)).append(",");

            velocityValues.add(velocity);
            avgVelocityValues.add(teamAverageVelocity);
            sprintNameValues.add(sprintName);

            // for scaling
            if (velocity > xMax) {
                xMax = (int) Math.ceil(velocity);
            }
        }

        // set up y-axis
        url.append("1:");
        for (int idx = 0; idx <= xMax; idx++) {
            url.append("|").append(idx);
        }

        // add velocity and average velocity data points
        String valuePoints = (values.length() > 0) ? values.substring(0, values.length() - 1) : "0";
        String averagePoints = (average.length() > 0) ? average.substring(0, average.length() - 1) : "0";
        url.append("&chd=t:").append(valuePoints).append("|").append(averagePoints);

        // scaling
        url.append("&chds=0,").append(xMax);

        return url.toString();
    }

    /**
     * @return the sprintTeamStats
     */
    public List<SprintTeamStats> getSprintTeamStats() {
        if (sprintTeamStats == null) {
            sprintTeamStats = ManagerUtils.getSprintTeamStatsManager().listByTeamId(getSprintTeam().getTeam().getId());
        }

        return sprintTeamStats;
    }

    /**
     * @return the sprintTeam
     */
    public SprintTeam getSprintTeam() {
        if (sprintTeam == null) {
            sprintTeam = ManagerUtils.getSprintTeamManager().get(id);
        }
        return sprintTeam;
    }
    
    public String getVelocityValues(){
        return listDoubleToJSON(velocityValues, "velocityValues");
    }
    
    public String getAvgVelocityValues(){
        return listDoubleToJSON(avgVelocityValues, "avgVelocityValues");
    }
    
    public String getSprintNameValues(){
        return listStringToJSON(sprintNameValues, "sprintNames");
    }
    
    private String listDoubleToJSON(List<Double> values, String jsonName) {
        StringBuilder retValue = new StringBuilder("var ");
        retValue.append(jsonName);
        retValue.append(" = [");;
        for( Double value : values){
            retValue.append(value.doubleValue());
            retValue.append(", ");
        }
        retValue.append(" ];");
        return retValue.toString();
    }
    
    private String listStringToJSON(List<String> values, String jsonName) {
        StringBuilder retValue = new StringBuilder("var ");
         retValue.append(jsonName);
         retValue.append(" = [");
        for( String value : values){
            retValue.append("\"");
            retValue.append(value);
            retValue.append("\"");
            retValue.append(", ");
        }
        retValue.append(" ];");
        return retValue.toString();   
    }
}
